@@ -1,16 +1,17 @@
 package CatalystX::ComponentsFromConfig::Role::AdaptorRole;
 {
-  $CatalystX::ComponentsFromConfig::Role::AdaptorRole::VERSION = '1.002';
+  $CatalystX::ComponentsFromConfig::Role::AdaptorRole::VERSION = '1.003';
 }
 {
   $CatalystX::ComponentsFromConfig::Role::AdaptorRole::DIST = 'CatalystX-ComponentsFromConfig';
 }
 use MooseX::Role::Parameterized;
 use Moose::Util::TypeConstraints;
+use Moose::Util 'with_traits';
 use MooseX::Types::Moose qw/ HashRef ArrayRef Str /;
 use MooseX::Types::Common::String qw/LowerCaseSimpleStr/;
 use MooseX::Types::LoadableClass qw/LoadableClass/;
-use MooseX::Traits::Pluggable 0.10;
+use Sub::Install;
 use namespace::autoclean;
 
 # ABSTRACT: parameterised role for trait-aware component adaptors
@@ -25,28 +26,6 @@ role {
     my $params = shift;
     my $type = ucfirst($params->component_type);
 
-    with 'MooseX::Traits::Pluggable' => {
-        -excludes => ['new_with_traits'],
-        -alias => { _build_instance_with_traits => 'build_instance_with_traits' },
-    };
-
-
-
-    method _trait_namespace => sub {
-        my ($self) = @_;
-        my $class = $self->class;
-        my $app_name = $self->app_name;
-        if ($class =~ s/^\Q$app_name//) {
-            my @list;
-            do {
-                push(@list, "${app_name}::TraitFor" . $class)
-            } while ($class =~ s/::\w+$//);
-            push(@list, "${app_name}::TraitFor::${type}" . $class);
-            return \@list;
-        }
-        return $class . '::TraitFor';
-    };
-
 
     has class => (
         isa => LoadableClass,
@@ -55,12 +34,6 @@ role {
         coerce => 1,
     );
 
-    has app_name => (
-        isa => 'Str',
-        is => 'rw',
-        init_arg => undef,
-    );
-
 
     has args => (
         isa => HashRef,
@@ -72,28 +45,51 @@ role {
     has traits => (
         isa => ArrayRef[Str],
         is => 'ro',
-        default => sub { [] },
     );
 
     around COMPONENT => sub {
         my ($orig, $class, $app, @rest) = @_;
 
         my $self = $class->$orig($app,@rest);
-        $self->app_name($app);
+        my $other_class = $self->class;
 
-        unless ($self->class->can('meta')) {
+        unless ($other_class->can('meta')) {
             Moose->init_meta(
-                for_class => $self->class,
+                for_class => $other_class,
             );
         }
 
-        $self->build_instance_with_traits(
-            $self->class,
-            {
+        if ($self->traits) {
+            if (not $other_class->can('new_with_traits')) {
+                my $new_class = with_traits(
+                    $other_class,
+                    'MooseX::Traits::Pluggable',
+                );
+
+                Sub::Install::install_sub({
+                    code => sub {
+                        my ($other_self) = @_;
+                        if ($other_class =~ s/^\Q$app//) {
+                            my @list;
+                            do {
+                                push(@list, "${app}::TraitFor" . $other_class)
+                            } while ($other_class =~ s/::\w+$//);
+                            push(@list, "${app}::TraitFor::${type}" . $other_class);
+                            return \@list;
+                        }
+                        return $other_class . '::TraitFor';
+                    },
+                    into => $new_class,
+                    as => '_trait_namespace',
+                });
+                $other_class = $new_class;
+            }
+            return $other_class->new_with_traits({
                 traits => $self->traits,
                 %{ $self->args },
-            },
-        );
+            });
+        }
+        return $other_class->new($self->args);
     };
 };
 
@@ -103,7 +99,7 @@ __END__
 
 =pod
 
-=encoding utf-8
+=encoding UTF-8
 
 =head1 NAME
 
@@ -111,7 +107,7 @@ CatalystX::ComponentsFromConfig::Role::AdaptorRole - parameterised role for trai
 
 =head1 VERSION
 
-version 1.002
+version 1.003
 
 =head1 DESCRIPTION
 
@@ -134,16 +130,19 @@ Hashref to pass to the constructor of the adapted class.
 
 =head2 C<traits>
 
-Arrayref of traits / roles to apply to the class we're adapting. These
-will be processed by C<_build_instance_with_traits> in
-L<MooseX::Traits::Pluggable> (like C<new_with_traits>, see also
-L<MooseX::Traits>).
+Arrayref of traits / roles to apply to the class we're adapting.
 
-=head1 METHODS
+If set, and the L</class> has a C<new_with_traits> constructor, it
+will be called with L</args> augmented by C<< traits => $traits >>. If
+L</class> does not have that constructor, a subclass will be created
+by applying L<MooseX::Traits::Pluggable> to L</class>, to get the
+C<new_with_traits> constructor.
 
-=head2 C<_trait_namespace>
-
-Used by L<MooseX::Traits::Pluggable>. Given a L</class> of
+If L</class> provides C<new_with_traits>, that constructor is expected
+to deal with converting whatever is in C<traits> into actual module
+names. If, instead, we have to inject L<MooseX::Traits::Pluggable>, we
+depend on its own name expansion, but we provide our own set of
+namspaces: given a L</class> of
 C<My::App::Special::Class::For::Things> loaded into the C<My::App>
 Catalyst application, the following namespaces will be searched for
 traits / roles:
